# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

# The following is so I know about arrays:
from Numeric import *
from scipy_base.fastumath import *
from shapetest import *
from graftypes import *
 
class Lines :
   """This class is for Gist and possibly other graphics packages;
   it creates a class of disjoint lines which can then be inserted
   into and drawn by a graph2d object.
   
   x = Lines ( <keyword arguments> ) will create a Lines object
   with coordinates and various other characteristics. The keyword
   arguments are:
       x0 = <sequence of floating point values>
       y0 = <sequence of floating point values>
       x1 = <sequence of floating point values>
       y1 = <sequence of floating point values>
          x0, y0, x1, and y1 can actually be scalars, but if arrays
          must match in size and shape. (x0[i], y0[i]) represents
          the starting point of the ith line, and (x1[i], y1[i])
          represents its endpoint.
       color = one of the legal values for Gist (currently the
          only package supporting Lines). See gist.help for details.
       hide = 0/1 (1 to hide this part of the graph)
       width = width of the lines. 1.0 (pretty narrow) is the default,
          I think successive values 2.0, 3.0, ... roughly represent
          width in pixels.
       type = "solid, "dash", "dot", "dashdot", "dashdotdot", and
          "none" (in which case the lines will be plotted as characters).
   """

   _LineSpecError = "LineSpecError"

   def type (self) :
       return LinesType

   def __init__ ( self , * kwds , ** keywords ) :
       if len (kwds) == 1 :
          keywords = kwds[0]
       if not keywords.has_key ("x0") or not keywords.has_key ("y0") or \
          not keywords.has_key ("x1") or not keywords.has_key ("y1" ) :
          raise self._LineSpecError , \
             "You need all four keywords x0, y0, x1, and y1."
       self.x0 = keywords ["x0"]
       self.y0 = keywords ["y0"]
       self.x1 = keywords ["x1"]
       self.y1 = keywords ["y1"]
       if len (self.x0) != len (self.y0) or len (self.x0) != len (self.x1) or \
          len (self.x0) != len (self.y1) or len (self.y0) != len (self.x1) or \
          len (self.y0) != len (self.y1) or len (self.x1) != len (self.y1) :
          raise self._LineSpecError , \
             "x0, y0, x1, and y1 must all be the same length."
       if keywords.has_key ("label" ) :
          self.label = keywords [ "label" ]
       else :
          self.label = " "
       if keywords.has_key ( "hide" ) :
          self.hide = keywords [ "hide" ]
       else :
          self.hide = 0
       if keywords.has_key ( "width" ) :
          self.width = keywords [ "width" ]
       else :
          self.width = 1.0
       if keywords.has_key ( "type" ) :
          self.line_type = keywords [ "type" ]
       else :
          self.line_type = "solid"
       if keywords.has_key ( "color" ) :
          self.color = keywords ["color"]
       else :
          self.color = "foreground" # foreground

   def new ( self, ** keywords ) :
       """ new (...keyword arguments...) allows you to reuse a
       previously existing curve.
       """
       self.__init__ ( keywords )

   def set ( self , ** keywords ) :
       """ set (...keyword arguments...) allows you to set individual
       curve characteristics. No error checking is done.
       """
       for k in keywords.keys ():
          if k == "x0" :
             self.x0 = keywords ["x0"]
          elif k == "y0" :
             self.y0 = keywords ["y0"]
          elif k == "x1" :
             self.x1 = keywords ["x1"]
          elif k == "y1" :
             self.y1 = keywords ["y1"]
          elif k == "type" :
             self.line_type = keywords ["type"]
          else :
             setattr (self, k, keywords [k])



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

from Numeric import *
from scipy_base.fastumath import *
GraphicsError = "GraphicsError"
import os
try:
   graphics = os.environ["PYGRAPH"]
except KeyError:
   graphics = "Gist"

if graphics [0:3] == "Nar" :
   import NarPlotter
elif graphics == "Gist" :
   import GistPlotter
else :
   raise GraphicsError , \
      graphics + " is an unknown graphics package. Check PYGRAPH " + \
         "environment variable."

from curve import Curve
from graph2d import *
def paws ( ) :
    i = raw_input ("Type in any string to continue; ^C to return to prompt. ")
    return
 
if graphics [0:3] == "Nar" :
   print "This is a test of the Python interface to the Limeil Lab graphics"
   print "package, Narcisse. You  need Narcisse to be running. Fire it up by"
   print "typing /dist/basis/Narcisse/bin/Narcisse."
elif graphics == "Gist" :
   print "This is a test of the Python interface to the Gist graphics package."
 
paws ()
 
vsf = 0.
c = 1
s = 1000.
kmax = 25
lmax = 35
 
c1 = Curve ( y = arange (1, kmax+1, typecode = Float) , color = "yellow" )
g2 = Graph2d ( c1 , titles = ["Bottom of the Barrel", "Top Dog",
                              "Leftist", "Reaganist"] )
g2.plot ( )
paws ( )
c2 = Curve ( y = sqrt (arange (1, kmax+1, typecode = Float)**3) ,
   color = "blue")
g2.add ( c2 )
g2.plot ( )
paws ( )
c1.set (marks = 1, marker = "A")
c2.set (marks = 1, marker = "B")
g2.plot ( )
paws ( )
c1.set (marks = 0)
c2.set (marks = 0)
c1.set (width = 3.0)
c2.set (width = 4.0)
g2.plot ( )
paws ( )
c1.set (type = "dash")
c2.set (type = "dashdot")
g2.plot ( )
paws ( )
g2.delete ( 1 )
g2.plot ( )
paws ( )
mycolor= ["red", "blue", "yellow", "green", "orange", "purple"]
markers= ["+", "*", "o", "x", "."]
c5 = [ ]
for i in range (len (markers)) :
   c5.append ( Curve (
      y = exp (array (multiply ((1.0+(i+1)/10.), log (range (1, kmax))))) ,
               color = mycolor [i] , marks = 1, marker = markers [i] ,
               type = "none" )
             )
g2.new ( c5 , text="Line style and grid test" , text_color="black" ,
          text_pos = array ( [.2, .8]) , text_size = 60 ,
          titles = "Figure 1", old_plotter = 1)
g2.plot ( )
paws ( )
g2.quick_plot ( grid_type = "wide" )
paws ( )
part1 = abs (array ( [-5, -4, -3, -2, -1, .0000000001] ) )
c1.new (  y = part1, x = arange (-5, 1, typecode = Float) ,
         type="line" , color = "blue" )
part2 = array ( [.0000000001, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10] )
c2.new ( y = part2 , x = arange (0, 11, typecode = Float) ,
         type = "line" , color = "blue" )
g2.new ( [c1, c2] , text = " " , axis_scales = "linlog" , titles = " " ,
          axis_limits = array ( [[-6., 10.], [1.e-6, 10.]]), old_plotter = 1)
g2.plot ( )
paws ( )
c1.new ( y = array (exp (multiply (log (10.0), range (-50, 1)))) ,
         x = arange (1, 52, typecode = Float) )
g2.new ( c1 , axis_scales = "loglog" ,
              axis_limits = array ( [[1., 100.], [1.e-8, 1.]]), old_plotter = 1)
g2.plot ( )
paws ( )
c1.new ( y = array ( [1.e-48, 1.e-21, 0., 0., 0.]) )
g2.new ( c1 , axis_scales = "linlin" ,
              axis_limits = array  ( [[0., 0.], [0., 0.]]), old_plotter = 1)
g2.plot ( )



import types
points = 0.0013000
inches = 72.27*points
def_linewidth = 0.5*points
def_msize = 10.0*points
cm = inches/2.54

ltype = {'none':0,
         'solid':1,
         'dash':2,
         'dot':3,
         'dashdot':4,
         'dashdotdot':5}
mtype = {'point':1,
         'plus':2,
         'asterisk':3,
         'circle':4,
         'cross':5}
tfont = {'bold':1,
         'italic':2,
         'courier':0,
         'times':4,
         'helvetica':8,
         'symbol':12,
         'newcentury':16}
fontnames = ['courier', 'times', 'helvetica', 'symbol', 'newcentury']
fontstyles = ['bold', 'italic']
tdirec = {'right':0,
          'up':1,
          'left':2,
          'down':3}
thalign = {'normal':0,
           'left':1,
           'center':2,
           'right':3}
tvalign = {'normal':0,
           'top':1,
           'cap':2,
           'half':3,
           'base':4,
           'bottom':5}
_ticks = {'none':0,
         'left':0x001,
         'below':0x001,
         'right':0x002,
         'above':0x002,
         'center':0x004,
         'in':0x008,
         'out':0x010,
         'leftlabel':0x020,
         'belowlabel':0x020,
         'rightlabel':0x040,
         'abovelabel':0x040,
         'fullgrid':0x080,
         'origin':0x100}
colors = {'bg':-1,
          'fg':-2,
          'black':-3,
          'white':-4,
          'red':-5,
          'green':-6,
          'blue':-7,
          'cyan':-8,
          'magenta':-9,
          'yellow':-10}

defsys = {'legend':0,
          'viewport':[0.19,0.60,0.44,0.85],
          'ticks':{
              'horiz':{
                  'nMajor':7.5,
                  'nMinor':50.0,
                  'logAdjMajor':1.2,
                  'logAdjMinor':1.2,
                  'nDigits':3,
                  'gridLevel':1,
                  'flags':0x033,
                  'tickOff':0.0007,
                  'labelOff':0.0182,
                  'tickLen':[0.0143,0.0091,0.0052,0.0026,0.0013],
                  'tickStyle':{
                      'color':-2,
                      'type':1,
                      'width':1.0},
                  'gridStyle':{
                      'color':-2,
                      'type':3,
                      'width':1.0},
                  'textStyle':{
                      'color':-2,
                      'font':0x08,
                      'height':0.0182,
                      'orient':0,
                      'alignH':0,
                      'alignV':0,
                      'opaque':0},
                  'xOver':0.395,
                  'yOver':0.370
                  },
              'vert':{
                  'nMajor':7.5,
                  'nMinor':50.0,
                  'logAdjMajor':1.2,
                  'logAdjMinor':1.2,
                  'nDigits':4,
                  'gridLevel':1,
                  'flags':0x033,
                  'tickOff':0.0007,
                  'labelOff':0.0182,
                  'tickLen':[0.0143,0.0091,0.0052,0.0026,0.0013],
                  'tickStyle':{
                      'color':-2,
                      'type':1,
                      'width':1.0},
                  'gridStyle':{
                      'color':-2,
                      'type':3,
                      'width':1.0},
                  'textStyle':{
                      'color':-2,
                      'font':0x08,
                      'height':0.0182,
                      'orient':0,
                      'alignH':0,
                      'alignV':0,
                      'opaque':0},
                  'xOver':0.150,
                  'yOver':0.370
                  },
              'frame':0,
              'frameStyle':{
                  'color':-2,
                  'type':1,
                  'width':1.0}
              }
          }

def getsys(color=-2,frame=1,labelsize=14, font='helvetica', ticks='solid', hticpos='left right', vticpos='below above'):
    if type(color) != types.IntType:
        color = colors[color]
    labels = labelsize * points
    fontlist = string.split(string.lower(font))
    found = 0
    for k in range(len(fontlist)):
        if fontlist[k] in fontnames:
            found = 1
            break

    if not found:
        print "Unknown font: %s" % font
        return

    Nfont = tfont[fontlist[k]]
    for k in range(len(fontstyles)):
        if fontstyles[k] in fontlist:
            Nfont = Nfont + tfont[fontstyles[k]]

    ticksty = ltype[ticks]
    if ticksty == 0:
        hflags = 0
        vflags = 0
    else:
        hflags = vflags = 0x030
        for pos in string.split(hticpos):
            try:
                hflags = hflags + _ticks[pos]
            except KeyError:
                pass
        for pos in string.split(vticpos):
            try:
                vflags = vflags + _ticks[pos]
            except KeyError:
                pass

    newsys = {'legend':0,
              'viewport':[0.19,0.60,0.44,0.85],
              'ticks':{
                  'horiz':{
                      'nMajor':7.5,
                      'nMinor':50.0,
                      'logAdjMajor':1.2,
                      'logAdjMinor':1.2,
                      'nDigits':3,
                      'gridLevel':1,
                      'flags':hflags,
                      'tickOff':0.0007,
                      'labelOff':0.0182,
                      'tickLen':[0.0143,0.0091,0.0052,0.0026,0.0013],
                      'tickStyle':{
                          'color':color,
                          'type':ticksty,
                          'width':1.0},
                      'gridStyle':{
                          'color':color,
                          'type':3,
                          'width':1.0},
                      'textStyle':{
                          'color':color,
                          'font':Nfont,
                          'height':labels,
                          'orient':0,
                          'alignH':0,
                          'alignV':0,
                          'opaque':0},
                      'xOver':0.395,
                      'yOver':0.370
                      },
                  'vert':{
                      'nMajor':7.5,
                      'nMinor':50.0,
                      'logAdjMajor':1.2,
                      'logAdjMinor':1.2,
                      'nDigits':4,
                      'gridLevel':1,
                      'flags':vflags,
                      'tickOff':0.0007,
                      'labelOff':0.0182,
                      'tickLen':[0.0143,0.0091,0.0052,0.0026,0.0013],
                      'tickStyle':{
                          'color':color,
                          'type':ticksty,
                          'width':1.0},
                      'gridStyle':{
                          'color':color,
                          'type':3,
                          'width':1.0},
                      'textStyle':{
                          'color':color,
                          'font':Nfont,
                          'height':labels,
                          'orient':0,
                          'alignH':0,
                          'alignV':0,
                          'opaque':0},
                      'xOver':0.150,
                      'yOver':0.370
                      },
                  'frame':frame,
                  'frameStyle':{
                      'color':color,
                      'type':1,
                      'width':1.0}
                  }
              }
    return newsys
    
import string, types

def sys2string(system,level=0):
    retstr=""
    spaces = 4*level
    for key in system.keys():
        keytype = type(system[key])
        if keytype is types.DictType:
            retstr = retstr+"%s%s={\n%s" % (' '*spaces,key,sys2string(system[key],level+1))
        elif keytype is types.ListType:
            retstr = retstr+"%s%s={%s},\n" % (' '*spaces,key,string.join(map(str,system[key]),','))
        else:
            retstr = retstr+"%s%s=%s,\n" % (' '*spaces,key,system[key])
    return retstr[:-2]+"\n%s},\n" % (' '*spaces)
            
def style2string(systemslist, landscape=0):
    retstr = "landscape = %d\n" % landscape
    num = 1
    if type(systemslist) is types.ListType:
        num = len(systemslist)
        if type(systemslist[0]) is not types.DictType:
            raise TypeError, "dict2string: first argument should be a dicitonary or a list"
    elif type(systemslist) is not types.DictType:
        raise TypeError, "dict2string: first argument should be a dictionary or a list"
    else:
        systemslist = [systemslist]

    if num > 1:
        retstr = "%s\ndefault = {\n%s}\n" % (retstr, sys2string(systemslist[0],level=1)[:-3])
        retstr = "%s\nsystem = { legend=0 }\n" % retstr
        for k in range(1,num):
            retstr = "%s\nsystem = {\n%s}\n" % (retstr, sys2string(systemslist[k],level=1)[:-3])
    else:
        retstr = "%s\nsystem = {\n%s}\n" % (retstr, sys2string(systemslist[0],level=1)[:-3])

    return retstr

def writestyle(name, systemslist, landscape=0):
    fid = open(name,'w')
    fid.write(style2string(systemslist,landscape))
    fid.close()
    return






"""
fitpack --- curve and surface fitting with splines

fitpack is based on a collection of Fortran routines DIERCKX
by P. Dierckx (see http://www.netlib.org/dierckx/) transformed
to double routines by Pearu Peterson.
"""
# Created by Pearu Peterson, June,August 2003

__all__ = [
    'UnivariateSpline',
    'InterpolatedUnivariateSpline',
    'LSQUnivariateSpline',

    'LSQBivariateSpline',
    'SmoothBivariateSpline']

import warnings
from scipy_base import zeros, Float, concatenate, alltrue
from scipy_distutils.misc_util import PostponedException

try: import dfitpack
except: dfitpack = PostponedException()

################ Univariate spline ####################

_curfit_messages = {1:"""
The required storage space exceeds the available storage space, as
specified by the parameter nest: nest too small. If nest is already
large (say nest > m/2), it may also indicate that s is too small.
The approximation returned is the weighted least-squares spline
according to the knots t[0],t[1],...,t[n-1]. (n=nest) the parameter fp
gives the corresponding weighted sum of squared residuals (fp>s).
""",
                    2:"""
A theoretically impossible result was found during the iteration
proces for finding a smoothing spline with fp = s: s too small.
There is an approximation returned but the corresponding weighted sum
of squared residuals does not satisfy the condition abs(fp-s)/s < tol.""",
                    3:"""
The maximal number of iterations maxit (set to 20 by the program)
allowed for finding a smoothing spline with fp=s has been reached: s
too small.
There is an approximation returned but the corresponding weighted sum
of squared residuals does not satisfy the condition abs(fp-s)/s < tol.""",
                    10:"""
Error on entry, no approximation returned. The following conditions
must hold:
xb<=x[0]<x[1]<...<x[m-1]<=xe, w[i]>0, i=0..m-1
if iopt=-1:
  xb<t[k+1]<t[k+2]<...<t[n-k-2]<xe"""
                    }

class UnivariateSpline:
    """ Univariate spline s(x) of degree k on the interval
    [xb,xe] calculated from a given set of data points
    (x,y).
    """

    def __init__(self, x, y, w=None, bbox = [None]*2, k=3, s=None):
        """
        Input:
          x,y   - 1-d sequences of data points (x must be
                  in strictly ascending order)

        Optional input:
          w          - positive 1-d sequence of weights
          bbox       - 2-sequence specifying the boundary of
                       the approximation interval.
                       By default, bbox=[x[0],x[-1]]
          k=3        - degree of the univariate spline.
          s          - positive smoothing factor defined for
                       estimation condition:
                         sum((w[i]*(y[i]-s(x[i])))**2) <= s
                       Default s=len(w) which should be a good value
                       if 1/w[i] is an estimate of the standard
                       deviation of y[i].
        """
        #_data == x,y,w,xb,xe,k,s,n,t,c,fp,fpint,nrdata,ier
        data = dfitpack.fpcurf0(x,y,k,w=w,
                                xb=bbox[0],xe=bbox[1],s=s)
        if data[-1]==1:
            # nest too small, setting to maximum bound
            data = self._reset_nest(data)
        self._data = data
        self._reset_class()

    def _reset_class(self):
        data = self._data
        n,t,c,k,ier = data[7],data[8],data[9],data[5],data[-1]
        self._eval_args = t[:n],c[:n-k-1],k
        if ier==0:
            # the spline returned has a residual sum of squares fp
            # such that abs(fp-s)/s <= tol with tol a relative
            # tolerance set to 0.001 by the program
            pass
        elif ier==-1:
            # the spline returned is an interpolating spline
            self.__class__ = InterpolatedUnivariateSpline
        elif ier==-2:
            # the spline returned is the weighted least-squares
            # polynomial of degree k. In this extreme case fp gives
            # the upper bound fp0 for the smoothing factor s.
            self.__class__ = LSQUnivariateSpline
        else:
            # error
            if ier==1:
                self.__class__ = LSQUnivariateSpline
            message = _curfit_messages.get(ier,'ier=%s' % (ier))
            warnings.warn(message)

    def _reset_nest(self, data, nest=None):
        n = data[10]
        if nest is None:
            k,m = data[5],len(data[0])
            nest = m+k+1 # this is the maximum bound for nest
        else:
            assert n<=nest,"nest can only be increased"
        t,c,fpint,nrdata = data[8].copy(),data[9].copy(),\
                           data[11].copy(),data[12].copy()
        t.resize(nest)
        c.resize(nest)
        fpint.resize(nest)
        nrdata.resize(nest)
        args = data[:8] + (t,c,n,fpint,nrdata,data[13])
        data = dfitpack.fpcurf1(*args)
        return data

    def set_smoothing_factor(self, s):
        """ Continue spline computation with the given smoothing
        factor s and with the knots found at the last call.
        """
        data = self._data
        if data[6]==-1:
            warnings.warn('smoothing factor unchanged for'
                          'LSQ spline with fixed knots')
            return
        args = data[:6] + (s,) + data[7:]
        data = dfitpack.fpcurf1(*args)
        if data[-1]==1:
            # nest too small, setting to maximum bound
            data = self._reset_nest(data)
        self._data = data
        self._reset_class()

    def __call__(self, x, nu=None):
        """ Evaluate spline (or its nu-th derivative) at positions x.
        Note: x can be unordered but the evaluation is
        more efficient if x is (partially) ordered.
        """
        if nu is None:
            return dfitpack.splev(*(self._eval_args+(x,)))
        return dfitpack.splder(nu=nu,*(self._eval_args+(x,)))

    def get_knots(self):
        """ Return the positions of (boundary and interior)
        knots of the spline.
        """
        data = self._data
        k,n = data[5],data[7]
        return data[8][k:n-k]

    def get_coeffs(self):
        """ Return spline coefficients."""
        data = self._data
        k,n = data[5],data[7]
        return data[9][:n-k-1]

    def get_residual(self):
        """ Return weighted sum of squared residuals of the spline
        approximation: sum ((w[i]*(y[i]-s(x[i])))**2)
        """
        return self._data[10]

    def integral(self, a, b):
        """ Return definite integral of the spline between two
        given points.
        """
        return dfitpack.splint(*(self._eval_args+(a,b)))

    def derivatives(self, x):
        """ Return all derivatives of the spline at the point x."""
        d,ier = dfitpack.spalde(*(self._eval_args+(x,)))
        assert ier==0,`ier`
        return d

    def roots(self):
        """ Return the zeros of the spline.

        Restriction: only cubic splines are supported by fitpack.
        """
        k = self._data[5]
        if k==3:
            z,m,ier = dfitpack.sproot(*self._eval_args[:2])
            assert ier==0,`ier`
            return z[:m]
        raise NotImplementedError,'finding roots unsupported for non-cubic splines'

class InterpolatedUnivariateSpline(UnivariateSpline):
    """ Interpolated univariate spline approximation."""
    def __init__(self, x, y, w=None, bbox = [None]*2, k=3):
        """
        Input:
          x,y   - 1-d sequences of data points (x must be
                  in strictly ascending order)

        Optional input:
          w          - positive 1-d sequence of weights
          bbox       - 2-sequence specifying the boundary of
                       the approximation interval.
                       By default, bbox=[x[0],x[-1]]
          k=3        - degree of the univariate spline.
        """
        #_data == x,y,w,xb,xe,k,s,n,t,c,fp,fpint,nrdata,ier
        self._data = dfitpack.fpcurf0(x,y,k,w=w,
                                      xb=bbox[0],xe=bbox[1],s=0)
        self._reset_class()

class LSQUnivariateSpline(UnivariateSpline):
    """ Weighted least-squares univariate spline approximation."""

    def __init__(self, x, y, t, w=None, bbox = [None]*2, k=3):
        """
        Input:
          x,y   - 1-d sequences of data points (x must be
                  in strictly ascending order)
          t     - 1-d sequence of the positions of user-defined
                  interior knots of the spline (t must be in strictly
                  ascending order and bbox[0]<t[0]<...<t[-1]<bbox[-1])

        Optional input:
          w          - positive 1-d sequence of weights
          bbox       - 2-sequence specifying the boundary of
                       the approximation interval.
                       By default, bbox=[x[0],x[-1]]
          k=3        - degree of the univariate spline.
        """
        #_data == x,y,w,xb,xe,k,s,n,t,c,fp,fpint,nrdata,ier
        xb=bbox[0]
        xe=bbox[1]
        if xb is None: xb = x[0]
        if xe is None: xe = x[-1]
        t = concatenate(([xb]*(k+1),t,[xe]*(k+1)))
        n = len(t)
        if not alltrue(t[k+1:n-k]-t[k:n-k-1] > 0):
            raise ValueError,\
                  'Interior knots t must satisfy Schoenberg-Whitney conditions'
        data = dfitpack.fpcurfm1(x,y,k,t,w=w,xb=xb,xe=xe)
        self._data = data[:-3] + (None,None,data[-1])
        self._reset_class()


################ Bivariate spline ####################

_surfit_messages = {1:"""
The required storage space exceeds the available storage space: nxest
or nyest too small, or s too small.
The weighted least-squares spline corresponds to the current set of
knots.""",
                    2:"""
A theoretically impossible result was found during the iteration
process for finding a smoothing spline with fp = s: s too small or
badly chosen eps.
Weighted sum of squared residuals does not satisfy abs(fp-s)/s < tol.""",
                    3:"""
the maximal number of iterations maxit (set to 20 by the program)
allowed for finding a smoothing spline with fp=s has been reached:
s too small.
Weighted sum of squared residuals does not satisfy abs(fp-s)/s < tol.""",
                    4:"""
No more knots can be added because the number of b-spline coefficients
(nx-kx-1)*(ny-ky-1) already exceeds the number of data points m:
either s or m too small.
The weighted least-squares spline corresponds to the current set of
knots.""",
                    5:"""
No more knots can be added because the additional knot would (quasi)
coincide with an old one: s too small or too large a weight to an
inaccurate data point.
The weighted least-squares spline corresponds to the current set of
knots.""",
                    10:"""
Error on entry, no approximation returned. The following conditions
must hold:
xb<=x[i]<=xe, yb<=y[i]<=ye, w[i]>0, i=0..m-1
If iopt==-1, then
  xb<tx[kx+1]<tx[kx+2]<...<tx[nx-kx-2]<xe
  yb<ty[ky+1]<ty[ky+2]<...<ty[ny-ky-2]<ye""",
                    -3:"""
The coefficients of the spline returned have been computed as the
minimal norm least-squares solution of a (numerically) rank deficient
system (deficiency=%i). If deficiency is large, the results may be
inaccurate. Deficiency may strongly depend on the value of eps."""
                    }


class BivariateSpline:
    """ Bivariate spline s(x,y) of degrees kx and ky on the rectangle
    [xb,xe] x [yb, ye] calculated from a given set of data points
    (x,y,z).
    """

    def get_residual(self):
        """ Return weighted sum of squared residuals of the spline
        approximation: sum ((w[i]*(z[i]-s(x[i],y[i])))**2)
        """
        return self.fp
    def get_knots(self):
        """ Return a tuple (tx,ty) where tx,ty contain knots positions
        of the spline with respect to x-, y-variable, respectively.
        The position of interior and additional knots are given as
          t[k+1:-k-1] and t[:k+1]=b, t[-k-1:]=e, respectively.
        """
        return self.tck[:2]
    def get_coeffs(self):
        """ Return spline coefficients."""
        return self.tck[2]
    def __call__(self,x,y,mth='array'):
        """ Evaluate spline at positions x,y."""
        if mth=='array':
            tx,ty,c = self.tck[:3]
            kx,ky = self.degrees
            z,ier = dfitpack.bispev(tx,ty,c,kx,ky,x,y)
            assert ier==0,'Invalid input: ier='+`ier`
            return z
        raise NotImplementedError

class SmoothBivariateSpline(BivariateSpline):
    """ Smooth bivariate spline approximation."""

    def __init__(self, x, y, z, w=None,
                 bbox = [None]*4, kx=3, ky=3, s=None, eps=None):
        """
        Input:
          x,y,z  - 1-d sequences of data points (order is not
                   important) 
        Optional input:
          w          - positive 1-d sequence of weights
          bbox       - 4-sequence specifying the boundary of
                       the rectangular approximation domain.
                       By default, bbox=[min(x,tx),max(x,tx),min(y,ty),max(y,ty)]
          kx,ky=3,3  - degrees of the bivariate spline.
          s          - positive smoothing factor defined for
                       estimation condition:
                         sum((w[i]*(z[i]-s(x[i],y[i])))**2) <= s
                       Default s=len(w) which should be a good value
                       if 1/w[i] is an estimate of the standard
                       deviation of z[i].
          eps        - a threshold for determining the effective rank
                       of an over-determined linear system of
                       equations. 0 < eps < 1, default is 1e-16.
        """
        xb,xe,yb,ye = bbox
        nx,tx,ny,ty,c,fp,wrk1,ier = dfitpack.surfit_smth(x,y,z,w,\
                                                         xb,xe,yb,ye,\
                                                         kx,ky,s=s,eps=eps,lwrk2=1)
        if ier in [0,-1,-2]: # normal return
            pass
        else:
            message = _surfit_messages.get(ier,'ier=%s' % (ier))
            warnings.warn(message)

        self.fp = fp
        self.tck = tx[:nx],ty[:ny],c[:(nx-kx-1)*(ny-ky-1)]
        self.degrees = kx,ky

class LSQBivariateSpline(BivariateSpline):
    """ Weighted least-squares spline approximation.
    """
    def __init__(self, x, y, z, tx, ty, w=None,
                 bbox = [None]*4,
                 kx=3, ky=3, eps=None):
        """
        Input:
          x,y,z  - 1-d sequences of data points (order is not
                   important) 
          tx,ty  - strictly ordered 1-d sequences of knots
                   coordinates. 
        Optional input:
          w          - positive 1-d sequence of weights
          bbox       - 4-sequence specifying the boundary of
                       the rectangular approximation domain.
                       By default, bbox=[min(x,tx),max(x,tx),min(y,ty),max(y,ty)]
          kx,ky=3,3  - degrees of the bivariate spline.
          eps        - a threshold for determining the effective rank
                       of an over-determined linear system of
                       equations. 0 < eps < 1, default is 1e-16. 
        """
        nx = 2*kx+2+len(tx)
        ny = 2*ky+2+len(ty)
        tx1 = zeros((nx,),Float)
        ty1 = zeros((ny,),Float)
        tx1[kx+1:nx-kx-1] = tx
        ty1[ky+1:ny-ky-1] = ty

        xb,xe,yb,ye = bbox
        tx1,ty1,c,fp,ier = dfitpack.surfit_lsq(x,y,z,tx1,ty1,w,\
                                               xb,xe,yb,ye,\
                                               kx,ky,eps,lwrk2=1)
        if ier>10:
            tx1,ty1,c,fp,ier = dfitpack.surfit_lsq(x,y,z,tx1,ty1,w,\
                                                   xb,xe,yb,ye,\
                                                   kx,ky,eps,lwrk2=ier)
        if ier in [0,-1,-2]: # normal return
            pass
        else:
            if ier<-2:
                deficiency = (nx-kx-1)*(ny-ky-1)+ier
                message = _surfit_messages.get(-3) % (deficiency)
            else:
                message = _surfit_messages.get(ier,'ier=%s' % (ier))
            warnings.warn(message)
        self.fp = fp
        self.tck = tx1,ty1,c
        self.degrees = kx,ky




"""
foo - Scipy module xxx submodule.
"""

__all__ = ['xxx_foo_bar','foo_gun']

def xxx_foo_bar():
    """
    Scipy module xxx global (see info_xxx.py) test function.
    """
    return "Hello from xxx_foo_bar"

def foo_gun():
    """
    Scipy module xxx test function.
    """
    return "Hello from foo_gun"




# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

# The following is so I know about arrays:
from Numeric import *
from scipy_base.fastumath import *
from shapetest import *
from graftypes import *
from arrayfns import *
from LinearAlgebra import *
from gist import *

class QuadMesh :
   """This class is for Gist and possibly other graphics packages;
   it creates a class consisting of a QuadMesh, which can then
   be given to and plotted by a graph2d object.

   q = QuadMesh ( <keyword arguments> ) will create a QuadMesh
   object. The keyword arguments are:
   y and x, matching two-dimensional sequences of floating point
     values. These arguments are required and give the coordinates of
     the nodes of the mesh.
     --OR--
     x, y, and z can also be given as one-dimensional and
     of equal lengths. (In this case, all three arguments are
     required.) Then this will be assumed to be a random
     distribution of points (x, y) and values z on those points.
     In this case, a regular QuadMesh will be created, with nx
     equally spaced x's and ny equally spaced y's, with an nx
     by ny array of z's interpolated from the originals by
     Hardy's multiquadric fit. nx and ny can be specified by
     keyword values, lacking which they will be determined from
     ireg (if it is present), otherwise they will be defaulted
     to 50. xmax, xmin, ymax, and ymin for the new mesh may
     be specified by the corresponding keywords, or otherwise will
     by default be created by the maxima and minima of the variables
     x and y.
   ireg, optional two-dimensional sequence of integer values with
     the same dimensions as x and y, giving positive region numbers
     for the cells of the mesh, zero where the mesh does not exist.
     the first row and column of ireg are always zero, since there
     are one fewer cells in each direction than there are nodes.
   boundary = 0/1 0: plot entire mesh; 1: plot only the boundary of
     the selected region. if ktype and ltype are not "none",
     then the boundary will be plotted, then the k and l lines
     with their own types.
   boundary_type, boundary_color: these matter only if boundary = 1,
     and tell how the boundary will be plotted and what its color
     will be.
   region = n if n = 0, plot entire mesh; if any other number, plot
     the region specified (according to the settings in ireg).
   regions: "all" or a number or a list of numbers. Gives which
     regions to plot. If absent, defaults to [region], or "all"
     if region is 0. If present, and region is present also, then
     region is ignored.
   inhibit = 0/1/2 1: do not plot the (x [, j], y [, j]) lines;
     2: do not plot the (x [i, ], y[i, ]) lines.
   tri, optional two-dimensional sequence of values with
     the same dimensions as ireg, triangulation array used for
     contour plotting.
   z = optional two-dimensional sequence of floating point
     values. (Unless x and y are one-dimensional, in which case it
     must be present and match them in size. See above under x and y.)
     z has the same shape as x and y. If z is present, the
     contours of z will be plotted (default: 8 contours unless
     levels specifies otherwise), or a filled mesh will be
     plotted if filled = 1. In the latter case, z may be one
     smaller than x and y in each direction, and represents
     a zone-centered quantity.
   levels = optional one-dimensional sequence of floating point
     values. If present, a list of the values of z at which you
     want contours.
   filled = 0/1 If 1, plot a filled mesh using the values of z.
     If z is not present, the mesh zones will be filled with the
     background color, which allows plotting of a wire frame.
   contours = 0/1 only pertinent when filled = 1; if contours ia
     also 1, then will draw contours as specified by type, width,
     color, and levels.
   edges, if nonzero when filled=1, draw a solid edge around
     each zone.
   ecolor, ewidth--the color and width of the mesh lines when
     filled = 1 and adges is nonzero.
   vx, vy optional two-dimensional sequences of floating point
     values. Has the same shape as x and y. If present, represents
     a vector field to be plotted on the mesh.
   scale = floating point value. When plotting a vector field,
     a conversion factor from the units of (vx, vy) to the
     units of (x, y). 
   z_scale = specifies "log", "lin", or "normal" for how z is to be plotted.
   ktype, ltype: can have the same values as type, and allow the
     k and l mesh lines to be plotted differently.
   #### eventually, we can add kcolor, lcolor, kwidth, lwidth ####
   type, color, width, label, hide, marks, marker as for
     curves.
   """

   def type (self) :
       return QuadMeshType

   _QuadMeshSpecError = "QuadMeshSpecError"

   def __init__ ( self , * kwds , ** keywords ) :
       if len (kwds) == 1 :
          keywords = kwds[0]
       if not keywords.has_key ("x") or not keywords.has_key ("y") :
          raise self._QuadMeshSpecError , \
             "A QuadMesh requires both x and y keywords."
       self.x = keywords ["x"]
       self.y = keywords ["y"]
       if len (self.x.shape) == 2 :
          self.dimsofx = 2
          if self.x.shape != self.y.shape :
             raise self._QuadMeshSpecError , \
                "x and y must have the same shape."
          self.nx = self.x.shape [0]
          self.ny = self.x.shape [1]
       elif len (self.x.shape) == len (self.y.shape) == 1:
          self.dimsofx = 1
          if len (x) != len (y) :
             raise self._QuadMeshSpecError , \
                "If x and y are one dimensional, their lengths must agree."
          if keywords.has_key ("nx") :
             nx = keywords ["nx"]
          else :
             nx = None
          if keywords.has_key ("ny") :
             ny = keywords ["ny"]
          else :
             ny = None
          if keywords.has_key ("xmax") :
             xmax = keywords ["xmax"]
          else :
             xmax = max (x)
          if keywords.has_key ("ymax") :
             ymax = keywords ["ymax"]
          else :
             ymax = max (y)
          if keywords.has_key ("xmin") :
             xmin = keywords ["xmin"]
          else :
             xmin = min (x)
          if keywords.has_key ("ymin") :
             ymin = keywords ["ymin"]
          else :
             ymin = min (y)
          if keywords.has_key ("rsq") :
             rsq = keywords ["rsq"]
          else :
             rsq = 4. * (xmax - xmin) * (ymax - ymin) / len (x)
       else :
          raise self._QuadMeshSpecError , \
             "can't handle x (shape " + `self.x.shape` + ") and y " + \
             "(shape " + `self.y.shape` + "."
       if keywords.has_key ("boundary") :
          self.boundary = keywords ["boundary"]
       else :
          self.boundary = 0
       if keywords.has_key ("boundary_type") :
          self.boundary_type = keywords ["boundary_type"]
       else :
          self.boundary_type = "solid"
       if keywords.has_key ("boundary_color") :
          self.boundary_color = keywords ["boundary_color"]
       else :
          self.boundary_color = "fg"
       if keywords.has_key ("inhibit") :
          self.inhibit = keywords ["inhibit"]
       else :
          self.inhibit = 0
       if keywords.has_key ("label") :
          self.label = keywords ["label"]
       else :
          self.label = " "
       if keywords.has_key ("hide") :
          self.hide = keywords ["hide"]
       else :
          self.hide = 0
       if self.boundary == 1 :
          self.ktype = self.ltype = "none"
       else :
          self.ktype = self.ltype = "solid"
       if keywords.has_key ("type") :
          self.line_type = keywords ["type"]
       else :
          self.line_type = "solid"
       if keywords.has_key ("ktype") :
          self.ktype = keywords ["ktype"]
       if keywords.has_key ("ltype") :
          self.ltype = keywords ["ltype"]
       if keywords.has_key ("width") :
          self.width = keywords ["width"]
       else :
          self.width = 1
       if keywords.has_key ("color") :
          self.color = keywords ["color"]
       else :
          self.color = "fg"
       if keywords.has_key ("region") :
          self.region = keywords ["region"]
       else :
          self.region = 0
       if keywords.has_key ("tri") :
          self.tri = keywords ["tri"]
          if shape (tri) != shape (x) :
             raise self._QuadMeshSpecError , \
                "tri, x, and y must have the same shape."
       else :
          n1 = shape (self.x) [0]
          n2 = shape (self.x) [1]
          self.tri = zeros ( (n1, n2))
       if keywords.has_key ("ireg") and keywords ["ireg"] is not None :
          self.ireg = keywords ["ireg"]
          if self.dimsofx == 2 :
             if self.ireg.shape != self.x.shape :
                raise self._QuadMeshSpecError , \
                   "ireg, x, and y must have the same shape."
          else: # dimsofx has to be 1
             if self.nx is None :
                self.nx = self.ireg.shape [0]
             if self.ny is None :
                self.ny = self.ireg.shape [1]
             if self.ireg.shape != (nx, ny) :
                raise self._QuadMeshSpecError , \
                   "shape of ireg must be nx by ny."
       else :
          if self.dimsofx == 2:
             self.ireg = array (self.x).astype (Int)
          else :
             if self.nx is None :
                self.nx = 50
             if self.ny is None :
                self.ny = 50
             self.ireg = array ( (self.nx, self.ny), Int)
          self.ireg [0:self.nx, 0] = 0
          self.ireg [0, 0:self.ny] = 0
          self.ireg [1:self.nx, 1:self.ny] = 1
       if keywords.has_key ("filled") :
          self.filled = keywords ["filled"]
       else :
          self.filled = 0
       if keywords.has_key ("edges") :
          self.edges = keywords ["edges"]
       else :
          self.edges = 0
       if keywords.has_key ("contours") :
          self.contours = keywords ["contours"]
       elif self.filled == 0 and self.edges == 0 :
          self.contours = 1
       else :
          self.contours = 0
       if keywords.has_key ("ewidth") :
          self.ewidth = keywords ["ewidth"]
       else :
          self.ewidth = 1
       if keywords.has_key ("ecolor") :
          self.ecolor = keywords ["ecolor"]
       else :
          self.ecolor = "fg"
       if keywords.has_key ("levels") :
          self.levels = keywords ["levels"]
       else :
          self.levels = None
       if keywords.has_key ("z") :
          self.z = keywords ["z"]
       else :
          self.z = None
       if keywords.has_key ("z_scale") and keywords ["z_scale"] is not None :
          self.z_scale = keywords ["z_scale"]
       else :
          self.z_scale = "lin"
       if self.dimsofx == 1 :
          if self.z is None or len (self.z.shape) != 1 \
             or len (self.z) != len (self.x) :
             raise self._QuadMeshSpecError , \
                "If x and y are one-dimensional, " + \
                "then z must be too, and must be the same length."
          # Compute regular mesh and interpolated values
          dx = (xmax - xmin) / nx
          dy = (ymax - ymin) / ny
          xcplot = span (xmin, xmax - dx, nx)
          ycplot = span (ymin, ymax - dy, ny)
          del xmin, xmax, ymin, ymax, dx, dy
          xt = subtract.outer (self.x, self.x)
          yt = subtract.outer (self.y, self.y)
          aa = sqrt (xt * xt + yt * yt + rsq)
          del xt, yt
          alpha = array (self.z, copy = 1)
          alpha = solve_linear_equations (aa, alpha)
          self.z = mfit (alpha, self.x, xcplot, self.y,
             ycplot, rsq)
          del aa, alpha, rsq
          # Expand coordinates to 2d arrays to match zcplot
          self.x = multiply.outer (xcplot, ones (ny, Float))
          self.y = multiply.outer (ones (nx, Float), ycplot)
          del xcplot, ycplot, nx, ny
       if self.dimsofx == 2 and self.z is not None : # check for shape
          if self.filled == 0 :
             if self.z.shape != self.x.shape :
                raise self._QuadMeshSpecError , \
                   "z, x, and y must be the same shape."
          else :
             n1 = self.z.shape [0]
             n2 = self.z.shape [1]
             m1 = self.x.shape [0]
             m2 = self.x.shape [1]
             if n1 == m1 and n2 == m2 or n1 == m1 - 1 and n2 == m2 - 1 :
                pass
             else :
                raise self._QuadMeshSpecError , \
                   "z, x, and y must be the same shape, or z one smaller" + \
                   " in each dimension."
       if keywords.has_key ("vx") and not is_scalar (keywords ["vx"]) and \
          len (keywords ["vx"]) > 0 :
          if not keywords.has_key ("vy") :
             raise self._QuadMeshSpecError , \
                "vx and vy must both be present."
          self.vx = keywords ["vx"]
          self.vy = keywords ["vy"]
       else :
          self.vx = None
          self.vy = None
       if keywords.has_key ("scale") :
          self.scale = keywords ["scale"]
       else :
          self.scale = None
       if keywords.has_key ("marks") :
          self.marks = keywords ["marks"]
       else :
          self.marks = 0
       if keywords.has_key ( "marker" ) :
          self.marker = keywords [ "marker" ]
       else :
          self.marker = "unspecified"
       if keywords.has_key ( "regions" ) :
          self.regions = keywords ["regions"]
          if is_scalar (self.regions) and self.regions != "all" :
             self.regions = [self.regions]
       else :
          if self.region == 0 :
             self.regions = "all"
          else:
             self.regions = [self.region]

   def new ( self, ** keywords ) :
       """ new (...keyword arguments...) allows you to reuse a
       previously existing quadmesh.
       """
       self.__init__ ( keywords )
 
   def set ( self , ** keywords ) :
       """ set (...keyword arguments...) allows you to set individual
       quadmesh characteristics. Very little error checking is done.
       """
       # The following allopws vx and vy to be cleared by sending in []
       if keywords.has_key ("vx") :
          self.vx = keywords ["vx"]
          del keywords ["vx"]
       if keywords.has_key ("vy") :
          self.vy = keywords ["vy"]
          del keywords ["vy"]
       for k in keywords.keys ():
          if k == "type" :
             self.line_type = keywords ["type"]
          else :
              setattr (self, k, keywords [k])



#!/usr/bin/env python
#
# Created by: Pearu Peterson, October 2003
#

import sys
from scipy_test.testing import *
set_package_path()
import linalg.atlas_version
del sys.path[0]

# No tests possible.
# Importing atlas_version will print out atlas version.



# Copyright (c) 1996, 1997, The Regents of the University of California.
# All rights reserved.  See Legal.htm for full text and disclaimer.

# Nar, Numeric, and shapetest are all imported by graph:

from types import *
from graph import *
from graftypes import *

class Graph3d ( Graph ) :

   """
   g = Graph3d ( <surface  list>, ...keyword arguments...) will create
   a three-dimensional graphics object consisting of several surfaces
   plus a global environment for the object. It will accept one or
   a list of Plotter objects or plotter_identifiers, or will try
   to complete a generic connection of its own if asked to plot
   without such a plotter specification.
   <surface list> is one or a sequence of Surface  objects.
   The keyword arguments for Graph3d are:

       plotter = <Plotter object> or a sequence of <Plotter object>s
              if you have a plotter object or a sequence of them that
              you want the curve to use when plotting itself. I
              recommend against this; a curve can create its own
              plotter if you don't give it one.
       filename = <string> or a sequence of <string>s if you want to
              connect to Narcisse with the named file(s). (The default
              is " ".) Only one of the keywords 'plotter' or 'filename'
              is allowed, and both are optional.
              NOTE: the possibility of a sequence of file names or
              plotters allows one to display Narcisse graphs on one's
              own machine as well as one or more remote machines.
              In Gist, the filename argument is the same as the
              display argument (below).
       display = <string> or a sequence of <string>s if you want to
              display on the named hosts. The form of <string> is
              the usual "hostname:server.screen". The purpose of
              this argument is to allow you to continue without
              exiting python if you have to open a Gist window
              without the DISPLAY environment variable having been
              set, or if you want to open Gist windows on more
              than one host.
       titles = <value> where <value> is a string or a sequence
                of up to four strings, giving the titles in the
                order bottom, top, left, right.
       title_colors = <value> where value is an integer or string
                or a sequence of up to four integers or strings giving
                the colors of the titles.
       grid_type = <string> where "none" means no axis grid;
                   "axes" means a pair of axes with tick marks;
                   "wide" means a widely spaced 2d grid; and
                   "full" means a closely spaced 2d grid.
       axis_labels = <value> where <value> is a string or
                   sequence of up to five strings representing
                   the labels of the x axis, the y axis, the z
                   axis, the c axis, and the right y axis.
                   For Gist, only the first three count, and
                   if necessary, will be truncated to a single
                   character.
       gnomon = (0|1) (Gist only). If 1, show the gnomon (a simple display
                   showing the orientation of the xyz axes of the object).
       x_axis_label, y_axis_label, z_axis_label, c_axis_label, and
                   yr_axis_label may be used to change individual
                   axis labels.
       axis_limits = <value> where <value> is a pair [xmin, xmax] or
                   a sequence of up to five pairs, where the second
                   would be the y limits, the third the z limits,
                   the fourth the c limits, and the fifth the right
                   y limits.
       x_axis_limits, y_axis_limits, z_axis_limits, c_axis_limits, and
                   yr_axis_limits may be used to change individual
                   axis limits.
       x_factor, y_factor (defaults 1.0). A factor > 1.0 compresses
                   the graph by that factor in the specified direction
                   (relative to the plane of the graph). A factor < 1.0
                   would enlarge it in the specified direction. So far
                   available only in Gist.
       axis_scales = <value> where <value> is a string or a sequence
                   of up to five strings, each of which is either "lin"
                   (linear scale) or "log" (logarithmic scale). Omitted
                   strings default to "lin". The order thay are
                   specified is x, y, z, c, and right y. If this
                   argument is missing, the existing axis scales
                   are unchanged.
       x_axis_scale, y_axis_scale, z_axis_scale, c_axis_scale, and
                   yr_axis_scale may be used to change individual
                   axis scales.
       text = <value> where value is one or a sequence of strings
                   representing texts to be placed on the plot.
       text_color = <value> where <value> is one or a sequence
                   of integer color numbers or strings (names of
                   common colors) giving colors for the texts.
       text_size = <value> where <value> is one or a sequence of
                   integers giving (roughly) the number of characters
                   in a line on the graph. The larger this number, the
                   smaller the size.
       text_pos = <value> where <value> is a pair or a sequence of
                   reals between 0. and 1.0 giving the relative
                   position of the lower left corner of a text
                   in the graphics window.
       Narcisse viewing angles:
          phi = <integer value> specifies the angle that the line from
                      the view point to the origin makes with the positive
                      z axis. The angle is in degrees. (default 45)
          theta = <integer value> specifies the angle made by the projection
                      of the line of view on the xy plane with the
                      positive x axis. The angle is in degrees. (default 45)
          roll = <integer value> specifies the angle of rotation of the
                      graph around the line from the origin to the view point.
                      The angle is measured in the positive direction,
                      i. e., if your right thumb is aligned outwards along
                      the line from the origin to the view point, then
                      your fingers curl in the positive direction.
                      The angle is in degrees. Not implemented for
                      Gist graphics. (default 0)
       Gist viewing angles:
          theta = <integer value> specifies the angle that the z axis
                makes with its projection onto the surface of the graph;
                positive if the z axis points out of the screen,
                negative if it points inwards. (default 30)
                If the z axis is in the screen, then the x and y axes
                will appear horizontal. Note: If the gnomon is turned on,
                then axes in the plane of the graphics window or pointing
                into it will have their labels highlighted.
          phi = <integer value> specifies the angle through which the
                x axis is rotated in the xy plane; positive if in the
                direction of the y axis, negative otherwise. (default -45)
       distance = <integer value> specifies the distance of the view
                   point from the origin. This is an integer between
                   0 and 20. 0 makes the distance infinite; otherwise
                   the smaller the number, the closer you are. This
                   number does not affect the size of the graph, but
                   rather the amount of distortion (on account of
                   perspective) in the picture (the closer you are,
                   the more distortion).
       link = <value> Used to link surfaces of different 3d options.
           normally all surfaces in a graph will have the same
           3d options. This value should be set to 1 if you want to
           graph two or more surfaces with different 3d options.
           otherwise multiple surface graphs will appear with the
           options of the last surface specified.
           Narcisse only.
       connect = <value> set to 1 for graphs of more than one surface
           to provide better hidden line removal. Must not be used
           with link.
           Narcisse only.
       sync = <value> set to 1 to synchronize with Narcisse before
           plotting the next graph. Keeps graphs sent in rapid
           succession from becoming garbled. Defaults to 1; set to
           0 if you don't have a timing problem.
           Narcisse only.
       Lighting variables (so far available only in Gist):
       ambient = <value> is a light level (arbitrary units)
           that is added to every surface independent of its orientation.
       diffuse = <value> is a light level which is proportional
           to cos(theta), where theta is the angle between the surface
           normal and the viewing direction, so that surfaces directly
           facing the viewer are bright, while surfaces viewed edge on are
           unlit (and surfaces facing away, if drawn, are shaded as if they
           faced the viewer).
       specular = <value> 
       spower = <value>
       sdir = <value> 
           specular = S_LEVEL is a light level proportional to a high
           power spower=N of 1+cos(alpha), where alpha is the angle between
           the specular reflection angle and the viewing direction.  The light
           source for the calculation of alpha lies in the direction XYZ (a
           3 element vector) in the viewer's coordinate system at infinite
           distance.  You can have ns light sources by making S_LEVEL, N, and
           XYZ (or any combination) be vectors of length ns (3-by-ns in the
           case of XYZ).
       z_scale = <real value>
           if not equal to 1, affects the depth in z.
       color_bar = 0 or 1 (1 enables plotting of a color bar on
                any graphs for which it is meaningful (colored contour
                plots, filled contour plots, cell arrays, filled
                meshes and polygons).
       color_bar_pos (ignored unless a color bar is actually plotted)
                is a 2d array [ [xmin, ymin], [xmax, ymax]] specifying
                where (in window coordinates) the diagonally opposite
                corners of the color bar are to be placed.
                (the z_contours_array and/or c_contours array can be used
                to control precisely what is shown in the color bar.)
                currently not implemented in Gist.
       split = 0 or 1 (default 1) If on, causes the palette to be split
                when both planes and isosurfaces are present in a graph,
                so that isosurfaces are shaded according to current
                light settings, while plane sections of the mesh are
                colored according to a specified function.
                Currently not implemented in Narcisse.
   """

   def type (self) :
       return Graph3dType

   _color_card_dict = { "absolute" : 0 , "binary" : 1 ,
   "bluegreen" : 2 , "default" : 3 , "negative" : 4 , "positive" : 5 ,
   "rainbow" : 6 , "rainbowhls" : 7 , "random" : 8 , "redblue" : 9 ,
   "redgreen" : 10 , "shifted" : 11 }

   _axes = ["x", "y", "z", "c", "yr"]

   _NotASurface = "NotASurface"
   _LinkError = "LinkError"
   _SurfaceSpecError = "SurfaceSpecError"
   _SurfaceChangeError = "SurfaceChangeError"

   # The following are actually used in Graph
   _limits_keywords = ["x_axis_limits", "y_axis_limits", "z_axis_limits",
                       "c_axis_limits", "yr_axis_limits"]
   _scale_keywords = ["x_axis_scale", "y_axis_scale", "z_axis_scale",
                      "c_axis_scale", "yr_axis_scale"]
   _label_keywords = ["x_axis_label", "y_axis_label", "z_axis_label",
                      "c_axis_label", "yr_axis_label"]
   _no_of_axes = 5

   def __init__ ( self , surface_list , *kwds , ** keywords ) :
      if len ( kwds ) == 1 :
         keywords = kwds[0]
      if is_scalar ( surface_list ) :
         if surface_list.type () != SurfaceType and \
            surface_list.type () != Mesh3dType and \
            surface_list.type () != Slice3dType :
           raise self._NotASurface , \
           "Attempt to initialize with something that is not a surface, " + \
            "slice,  or mesh."
         self._s = [surface_list]
      else :
         for i in range ( len ( surface_list ) ) :
            if surface_list[i].type () != SurfaceType and \
               surface_list[i].type () != Mesh3dType and \
               surface_list[i].type () != Slice3dType :
               raise self._NotASurface , \
               "Attempt to initialize with something that is not a surface, " \
               + "slice,  or mesh."
         self._s = surface_list
      self._s_ln = len ( self._s )
      if keywords.has_key ("x_factor") :
         self._x_factor = keywords ["x_factor"]
      else :
         self._x_factor = 1.0
      if keywords.has_key ("y_factor") :
         self._y_factor = keywords ["y_factor"]
      else :
         self._y_factor = 1.0
      if keywords.has_key ("link") :
         if keywords.has_key ("connect") :
            raise _LinkError , "Can't specify both link and connect."
         self._link = keywords ["link"]
      else :
         self._link = 0
      if keywords.has_key ("connect") :
         self._connect = keywords ["connect"]
      else :
         self._connect = 0
      if keywords.has_key ("phi") :
         self._phi = keywords ["phi"]
      else :
         self._phi = None
      if keywords.has_key ("theta") :
         self._theta = keywords ["theta"]
      else :
         self._theta = None
      if keywords.has_key ("roll") :
         self._roll = keywords ["roll"]
      else :
         self._roll = None
      if keywords.has_key ("distance") :
         self._distance = keywords ["distance"]
      else :
         self._distance = 0
      if keywords.has_key ("z_scale") :
         self.z_scale = keywords ["z_scale"]
      else :
         self.z_scale = 1.0
      if keywords.has_key ("gnomon") :
         self._gnomon = keywords ["gnomon"]
      else :
         self._gnomon = None
      if keywords.has_key ( "color_card" ) :
         self._color_card = keywords ["color_card"]
      else :
         self._color_card = "default"
      self._axis_limits = [[0., 0.], [0., 0.], [0., 0.], [0., 0.], [0., 0.]]
      self._axis_scales = ["lin", "lin", "lin", "lin", "lin"]
      self._axis_labels = ["X axis", "Y axis", "Z axis", "C axis", "YR axis"]
      # Gist lighting
      self.lighting_dict = {}
      if keywords.has_key ("ambient") :
         self.lighting_dict ["ambient"] = keywords ["ambient"]
      else :
         self.lighting_dict ["ambient"] = None
      if keywords.has_key ("diffuse") :
         self.lighting_dict ["diffuse"] = keywords ["diffuse"]
      else :
         self.lighting_dict ["diffuse"] = None
      if keywords.has_key ("specular") :
         self.lighting_dict ["specular"] = keywords ["specular"]
      else :
         self.lighting_dict ["specular"] = None
      if keywords.has_key ("spower") :
         self.lighting_dict ["spower"] = keywords ["spower"]
      else :
         self.lighting_dict ["spower"] = None
      if keywords.has_key ("sdir") :
         self.lighting_dict ["sdir"] = keywords ["sdir"]
      else :
         self.lighting_dict ["sdir"] = None
      if keywords.has_key ("style") :
         self._style = keywords ["style"]
      else :
         self._style = "nobox.gs"
      if keywords.has_key ("grid_type") :
         self._grid_type = keywords ["grid_type"]
      else :
         self._grid_type = "none"
      if keywords.has_key ( "opt_3d" ) :
         self.opt_3d = keywords ["opt_3d"]
      else :
         self.opt_3d = "wm"
      if keywords.has_key ( "mask" ) :
         self.mask = keywords ["mask"]
      else :
         self.mask = "max"
      if keywords.has_key ( "split" ) :
         self._split = keywords ["split"]
      else :
         self._split = 0
      # Everything else is Graph generic:
      self._label_type = " "
      Graph.__init__ ( self , keywords )

   def new ( self , surface_list , ** keywords ) :
      """new ( surface_list, <keyword arguments> ) cleans out a Graph3d
      and reinitializes it. This has the same argument list as
      Graph3d. Do not change the plotter list or filename list (to
      avoid the tedious on-and-off flickering of windows) unless
      this is actually requested.
      """
      pl = self._plotter_list
      fl = self._filename_list
      self._plotter_list = []
      self._filename_list = []
      del self._s, self._s_ln, self._link, self._connect, self._phi, \
          self._theta, self._roll, self._distance, self._titles, \
          self._title_colors, self._text, self._text_color, self._text_size, \
          self._sync, self._grid_type, self._axis_labels, self._axis_scales, \
          self._axis_limits
      self.__init__ ( surface_list , keywords )
      if self._plotter_list == [] :
         self._plotter_list = pl
         self._filename_list = fl

   def add ( self , surface ) :
      """ add ( surface ) adds a surface with its characteristics to the
      existing plot. Surfaces are numbered in the order that they are
      added, beginning with 1.
      """
      if surface.type () != SurfaceType and surface.type () != Mesh3dType and \
         surface_list[i].type () != Slice3dType :
         raise self._NotASurface , \
         "Attempt to add something not a surface, slice, or mesh."
      self._s.append ( surface )
      self._s_ln = len (self._s)

   def delete ( self , n ) :
      """delete ( n ) deletes the nth surface from the Graph. Note
      that surfaces are numbered beginning with 1.
      """
      DeleteError = "DeleteError"
      if 1 <= n <= self._s_ln :
         self._s [n-1:n] = []
         self._s_ln = len (self._s)
      else :
         raise DeleteError , "There is no surface numbered " + `n` + \
            " in the current graph, which has only " + `self._s_ln` + \
            " surfaces."

   def set_surface_list (self, surface_list) :
      del self._s
      self._s = surface_list
      self._s_ln = len (surface_list)
      return

   def replace ( self , n , surface ) :
      """replace ( n , surface ) replaces the nth surface in the Graph
      with the specified surface. Note that surfaces are numbered
      beginning with 1.
      """
      if surface.type () != SurfaceType and surface.type () != Mesh3dType and \
               surface.type () != Slice3dType :
         raise self._NotASurface , \
            "Attempt to add something not a surface, slice, or mesh."
      ReplaceError = "ReplaceError"
      if 1 <= n <= self._s_ln :
         self._s [n-1] = surface
      else :
         raise ReplaceError , "There is no surface numbered " + `n` + \
            " in the surfacent graph, which has only " + `self._s_ln` + \
            " surfaces."

   def change_plot ( self , ** keywords ) :
      """change_plot ( <keyword arguments> ) is used to change any Graph3d
      characteristics except the surfaces being graphed. Use the add,
      delete, and/or replace commands to do that. change_plot will
      draw the graph without sending surface coordinates, unless
      keyword send is 1. Generally, change_plot should be used when
      the graph needs to be recomputed, and quick_plot when it does not.
      change_plot does no error checking and does not convert user-friendly
      names of colors and such into numbers.
      """
      for k in keywords.keys ():
         if k == "surface" or k == "mesh" :
            raise self._SurfaceChangeError, \
               "Use add, delete, or replace to change surfaces in a graph."
         setattr (self, "_" + k, keywords [k])
      if "send" in keywords.keys ():
         send = keywords ["send"]
      else:
         send = 0
      self._send_coordinates = send
      self.plot ( )
      self._send_coordinates = 1

   def quick_plot ( self , ** keywords ) :
      """quick_plot ( <keyword arguments> ) is used to change some Graph3d
      characteristics which do not demand that the graph be recomputed.
      You can change the characteristics of a surface in the graph by
      specifying its number (surface = n) and any combination of the
      traits color_card, opt_3d, mesh_type, or mask. Or you can change
      such overall graph characteristics as  titles, title_colors, text,
      text_color, text_size, text_pos, color_card, grid_type, sync, theta,
      phi, roll, and axis_labels.
      Or you can do both.
      The changes will be effected and the graph redrawn.
      Things that you cannot change include axis limits and scales,
      and the coordinates of a surface. Use change_plot if axis limits
      and scales are among the things you want to change, and use add,
      delete, or replace followed by a call to plot, if you wish to
      change a surface.
      quick_plot will not work right for link'ed surfaces. Once the
      changes have been made, you will have to call plot.
      """
      ChangeError = "ChangeError"
      PlottersNotStarted = "PlottersNotStarted"
      if not self._plotters_started :
         raise PlottersNotStarted , \
            "quick_plot requires that all plotters have already been started."
      if keywords.has_key ("opt_3d") :
         self._opt_3d = keywords ["opt_3d"]
         self.opt_3d_change = 1
      if keywords.has_key ("mesh_type" ) :
         self._mesh_type = keywords ["mesh_type"]
         self.mesh_type_change = 1
      if keywords.has_key ("mask") :
         self._mask = keywords ["mask"]
         self.mask_change = 1
      if keywords.has_key ("color_card") :
         if type ( keywords [ "color_card" ] ) == IntType :
            self._color_card = keywords [ "color_card" ]
            self.color_card_change = 1
         elif self._color_card_dict.has_key \
            ( keywords [ "color_card" ] ) :
            self._color_card = \
               self._color_card_dict [keywords [ "color_card" ]]
            self.color_card_change = 1
      if keywords.has_key ( "surface" ) or keywords.has_key ( "mesh" ) :
         if keywords.has_key ( "surface" ) :
            self.n = keywords ["surface"]
         else :
            self.n = keywords ["mesh"]
         self.color_card_change = 0
         self.opt_3d_change = 0
         self.mesh_type_change = 0
         self.mask_change = 0
         if 1 <= self.n <= self._s_ln :
            if keywords.has_key ("color_card") :
               if type ( keywords [ "color_card" ] ) == IntType :
                  self._s [self.n - 1].color_card = keywords [ "color_card" ]
                  self.color_card_change = 1
               elif self._color_card_dict.has_key \
                  ( keywords [ "color_card" ] ) :
                  self._s [self.n - 1].color_card = \
                     self._color_card_dict [keywords [ "color_card" ]]
                  self.color_card_change = 1
         else :
            raise ChangeError , "There is no surface numbered " + `n` + \
            " in the current graph, which has only " + `self._s_ln` + \
            " surfaces."
      if keywords.has_key ("grid_type") :
         self._grid_type = keywords ["grid_type"]
      if keywords.has_key ("sync") :
         self._sync = keywords ["sync"]
      if keywords.has_key ("theta") :
         self._theta = keywords ["theta"]
      if keywords.has_key ("phi") :
         self._phi = keywords ["phi"]
      if keywords.has_key ("roll") :
         self._roll = keywords ["roll"]
      if keywords.has_key ("ambient") :
         self.lighting_dict ["ambient"] = keywords ["ambient"]
      else :
         self.lighting_dict ["ambient"] = None
      if keywords.has_key ("diffuse") :
         self.lighting_dict ["diffuse"] = keywords ["diffuse"]
      else :
         self.lighting_dict ["diffuse"] = None
      if keywords.has_key ("specular") :
         self.lighting_dict ["specular"] = keywords ["specular"]
      else :
         self.lighting_dict ["specular"] = None
      if keywords.has_key ("spower") :
         self.lighting_dict ["spower"] = keywords ["spower"]
      else :
         self.lighting_dict ["spower"] = None
      if keywords.has_key ("sdir") :
         self.lighting_dict ["sdir"] = keywords ["sdir"]
      else :
         self.lighting_dict ["sdir"] = None
      Graph.change ( self , keywords ) # Change generic traits
      for ipl in range (len (self._plotter_list)) :
         pl = self._plotter_list [ipl]
         pl.quick_plot (self)

   def plot ( self ) :
       """plot ( ) plots a 3d graph object. If the user has not by
       now specified plotter(s) or filename(s) then a generic plotter
       object will be created, if it is possible to find a local
       Graphics routine.
       """
       self._init_plotter ( )
       # grid_type means something different on 3d graphs
       for ipl in range (len (self._plotter_list)) :
          pl = self._plotter_list [ipl]
          pl.plot3d (self)
   
   def move_light_source (self, ** keywords) :
       """move_light_source (nframes = 30, angle = 12) causes a
       movie of the surrent graph to be shown, with the light source
       rotating through angle degrees each frame, for a total
       of nframes. If angle is not specified and nframes is,
       then the angle defaults to 360 / nframes.
       """
       if not keywords.has_key ("angle") and keywords.has_key ("nframes") :
          nframes = keywords ["nframes"]
          angle = 2 * pi / nframes
       else :
          if keywords.has_key ("nframes") :
             nframes = keywords ["nframes"]
          else :
             nframes = 30
          if keywords.has_key ("angle") :
             angle = keywords ["angle"]
          else :
             angle = 2 * pi / nframes
       for ipl in range (len (self._plotter_list)) :
          pl = self._plotter_list [ipl]
          pl.move_light_source (self, angle, nframes)

   def rotate (self, ** keywords) :
       """rotate (axis = array ([-1., 1., 0.],  Float), angle = 12,
       nframes = 30) Causes the current graph to be rotated about
       the specified axis, through the specified angle each frame,
       for a total of nframes frames. If angle is not specified and
       nframes is, then the angle defaults to 360 / nframes.
       """
       if keywords.has_key ("axis") :
          axis = keywords ["axis"]
       else :
          axis = array ([-1., 1., 0.],  Float)
       if not keywords.has_key ("angle") and keywords.has_key ("nframes") :
          nframes = keywords ["nframes"]
          angle = 2 * pi / nframes
       else :
          if keywords.has_key ("nframes") :
             nframes = keywords ["nframes"]
          else :
             nframes = 30
          if keywords.has_key ("angle") :
             angle = pi * keywords ["angle"] / 180. # angle is degrees
          else :
             angle = 2 * pi / nframes
       for ipl in range (len (self._plotter_list)) :
          pl = self._plotter_list [ipl]
          pl.rotate_graph (self, axis, angle, nframes)



#!/usr/bin/env python
# Created by Pearu Peterson, Aug 2003
""" Test xplt based demos for interpolate.fitpack2 module
"""
__usage__ = """
Build interpolate:
  python setup_interpolate.py build
Run demos (assumes that scipy is installed):
  python -i tests/demos_xplt.py
"""

import sys
from scipy_test.testing import set_package_path
set_package_path()
from interpolate.fitpack2 import UnivariateSpline,LSQUnivariateSpline,\
     InterpolatedUnivariateSpline
from interpolate.fitpack2 import LSQBivariateSpline, SmoothBivariateSpline
del sys.path[0]

from scipy import *

def demo1():
    x = arange(0,2*pi+pi/4,2*pi/8)
    xnew = arange(-pi/10,2*pi+pi/4+pi/10,pi/50)
    y = sin(x)


    def make_plot():
        xplt.plot(x,y,'x',xnew,spline(xnew),x,y,'b',xnew,sin(xnew),
                  spline.get_knots(),spline(spline.get_knots()),'o')        

    spline = UnivariateSpline(x,y,k=1)
    assert isinstance(spline,LSQUnivariateSpline)
    print 'Linear LSQ approximation of sin(x):',spline.__class__.__name__
    make_plot()
    print 'Residual=',spline.get_residual()
    raw_input('Press any key to continue..')

    spline.set_smoothing_factor(0)
    assert isinstance(spline,InterpolatedUnivariateSpline)
    print 'Linear interpolation of sin(x):',spline.__class__.__name__
    make_plot()
    print 'Residual=',spline.get_residual()
    raw_input('Press any key to continue..')

    spline = UnivariateSpline(x,y,k=1,s=0.1)
    print 'Linear smooth approximation of sin(x):',spline.__class__.__name__
    assert isinstance(spline,UnivariateSpline)
    make_plot()
    print 'Residual=',spline.get_residual()
    raw_input('Press any key to continue..')
    
if __name__ == "__main__":
    demo1()



"""
    base_info holds classes that define the information
    needed for building C++ extension modules for Python that
    handle different data types.  The information includes
    such as include files, libraries, and even code snippets.
    
    base_info -- base class for cxx_info, blitz_info, etc.                  
    info_list -- a handy list class for working with multiple
                 info classes at the same time.            
"""
import os
import UserList

class base_info:
    _warnings =[]
    _headers = []
    _include_dirs = []
    _libraries = []
    _library_dirs = []
    _support_code = []
    _module_init_code = []
    _sources = []
    _define_macros = []
    _undefine_macros = []
    _extra_compile_args = []
    _extra_link_args = []
    compiler = ''
    def set_compiler(self,compiler):
        self.check_compiler(compiler)
        self.compiler = compiler
    # it would probably be better to specify what the arguments are
    # to avoid confusion, but I don't think these classes will get
    # very complicated, and I don't really know the variety of things
    # that should be passed in at this point.
    def check_compiler(self,compiler):
        pass        
    def warnings(self):   
        return self._warnings
    def headers(self):   
        return self._headers
    def include_dirs(self):
        return self._include_dirs
    def libraries(self):
        return self._libraries
    def library_dirs(self):
        return self._library_dirs
    def support_code(self):
        return self._support_code
    def module_init_code(self):
        return self._module_init_code
    def sources(self):
        return self._sources
    def define_macros(self):
        return self._define_macros
    def undefine_macros(self):
        return self._undefine_macros
    def extra_compile_args(self):
        return self._extra_compile_args
    def extra_link_args(self):
        return self._extra_link_args        
        
class custom_info(base_info):
    def __init__(self):
        self._warnings =[]
        self._headers = []
        self._include_dirs = []
        self._libraries = []
        self._library_dirs = []
        self._support_code = []
        self._module_init_code = []
        self._sources = []
        self._define_macros = []
        self._undefine_macros = []
        self._extra_compile_args = []
        self._extra_link_args = []

    def add_warning(self,warning):
        self._warnings.append(warning)
    def add_header(self,header):
        self._headers.append(header)
    def add_include_dir(self,include_dir):
        self._include_dirs.append(include_dir)
    def add_library(self,library):
        self._libraries.append(library)
    def add_library_dir(self,library_dir):
        self._library_dirs.append(library_dir)
    def add_support_code(self,support_code):
        self._support_code.append(support_code)
    def add_module_init_code(self,module_init_code):
        self._module_init_code.append(module_init_code)
    def add_source(self,source):
        self._sources.append(source)
    def add_define_macro(self,define_macro):
        self._define_macros.append(define_macro)
    def add_undefine_macro(self,undefine_macro):
        self._undefine_macros.append(undefine_macro)    
    def add_extra_compile_arg(self,compile_arg):
        return self._extra_compile_args.append(compile_arg)
    def add_extra_link_arg(self,link_arg):
        return self._extra_link_args.append(link_arg)        

class info_list(UserList.UserList):
    def get_unique_values(self,attribute):
        all_values = []        
        for info in self:
            vals = eval('info.'+attribute+'()')
            all_values.extend(vals)
        return unique_values(all_values)

    def extra_compile_args(self):
        return self.get_unique_values('extra_compile_args')
    def extra_link_args(self):
        return self.get_unique_values('extra_link_args')
    def sources(self):
        return self.get_unique_values('sources')    
    def define_macros(self):
        return self.get_unique_values('define_macros')
    def sources(self):
        return self.get_unique_values('sources')
    def warnings(self):
        return self.get_unique_values('warnings')
    def headers(self):
        return self.get_unique_values('headers')
    def include_dirs(self):
        return self.get_unique_values('include_dirs')
    def libraries(self):
        return self.get_unique_values('libraries')
    def library_dirs(self):
        return self.get_unique_values('library_dirs')
    def support_code(self):
        return self.get_unique_values('support_code')
    def module_init_code(self):
        return self.get_unique_values('module_init_code')

def unique_values(lst):
    all_values = []        
    for value in lst:
        if value not in all_values or value == '-framework':
            all_values.append(value)
    return all_values




"""
unixccompiler - can handle very long argument lists for ar.
"""

import os
import sys
import new

from distutils.errors import DistutilsExecError, LinkError, CompileError
from distutils.unixccompiler import *


import log

# Note that UnixCCompiler._compile appeared in Python 2.3
def UnixCCompiler__compile(self, obj, src, ext, cc_args, extra_postargs, pp_opts):
    display = '%s: %s' % (os.path.basename(self.compiler_so[0]),src)
    try:
        self.spawn(self.compiler_so + cc_args + [src, '-o', obj] +
                   extra_postargs, display = display)
    except DistutilsExecError, msg:
        raise CompileError, msg
UnixCCompiler._compile = new.instancemethod(UnixCCompiler__compile,
                                            None,
                                            UnixCCompiler)


def UnixCCompile_create_static_lib(self, objects, output_libname,
                                   output_dir=None, debug=0, target_lang=None):
    objects, output_dir = self._fix_object_args(objects, output_dir)

    output_filename = \
                    self.library_filename(output_libname, output_dir=output_dir)
        
    if self._need_link(objects, output_filename):
        self.mkpath(os.path.dirname(output_filename))
        tmp_objects = objects + self.objects
        while tmp_objects:
            objects = tmp_objects[:50]
            tmp_objects = tmp_objects[50:]
            display = '%s: adding %d object files to %s' % (os.path.basename(self.archiver[0]),
                                               len(objects),output_filename)
            self.spawn(self.archiver + [output_filename] + objects,
                       display = display)

        # Not many Unices required ranlib anymore -- SunOS 4.x is, I
        # think the only major Unix that does.  Maybe we need some
        # platform intelligence here to skip ranlib if it's not
        # needed -- or maybe Python's configure script took care of
        # it for us, hence the check for leading colon.
        if self.ranlib:
            display = '%s:@ %s' % (os.path.basename(self.ranlib[0]),
                                   output_filename)
            try:
                self.spawn(self.ranlib + [output_filename],
                           display = display)
            except DistutilsExecError, msg:
                raise LibError, msg
    else:
        log.debug("skipping %s (up-to-date)", output_filename)
    return

UnixCCompiler.create_static_lib = \
  new.instancemethod(UnixCCompile_create_static_lib,
                     None,UnixCCompiler)




# Iterative methods using reverse-communication raw material
#   These methods solve
#   Ax = b  for x

#   where A must have A.matvec(x,*args) defined
#    or be a numeric array


__all__ = ['bicg','bicgstab','cg','cgs','gmres','qmr'] 
import _iterative
import scipy_base as sb

_type_conv = {'f':'s', 'd':'d', 'F':'c', 'D':'z'}

class get_matvec:
    methname = 'matvec'
    def __init__(self, obj, *args):
        self.obj = obj
        self.args = args
        if isinstance(obj, sb.ArrayType):
            self.callfunc = self.type1
            return
        meth = getattr(obj,self.methname,None)
        if not callable(meth):
            raise ValueError, "Object must be an array "\
                  "or have a callable %s attribute." % (self.methname,)

        self.obj = meth
        self.callfunc = self.type2

    def __call__(self, x):
        return self.callfunc(x)

    def type1(self, x):
        return sb.dot(self.obj, x)

    def type2(self, x):
        return self.obj(x,*self.args)

class get_rmatvec(get_matvec):
    methname = 'rmatvec'
    def type1(self, x):
        return sb.dot(x, self.obj)

class get_psolve:
    methname = 'psolve'
    def __init__(self, obj, *args):
        self.obj = obj
        self.args = args
        meth = getattr(obj,self.methname,None)   
        if meth is None:  # no preconditiong available
            self.callfunc = self.type1
            return

        if not callable(meth):
            raise ValueError, "Preconditioning method %s "\
                  "must be callable." % (self.methname,)

        self.obj = meth
        self.callfunc = self.type2

    def __call__(self, x):
        return self.callfunc(x)
    
    def type1(self, x):
        return x

    def type2(self, x):
        return self.obj(x,*self.args)

class get_rpsolve(get_psolve):
    methname = 'rpsolve'

class get_psolveq(get_psolve):

    def __call__(self, x, which):
        return self.callfunc(x, which)

    def type1(self, x, which):
        return x

    def type2(self, x, which):
        return self.obj(x,which,*self.args)

class get_rpsolveq(get_psolveq):
    methname = 'rpsolve'
    
def bicg(A,b,x0=None,tol=1e-5,maxiter=None):
    """Use BIConjugate Gradient iteration to solve A x = b

    Inputs:

    A --   An array or an object with matvec(x) and rmatvec(x) methods
           to represent A * x and A^H * x respectively.  May also have
           psolve(b) and rpsolve(b) methods for representing solutions
           to the preconditioning equations M * x = b and
           M^H * x = b respectively.
    b --   An n-length vector

    Outputs:

    x  --  The converged solution
    info -- output result
            0  : successful exit
            >0 : convergence to tolerance not achieved, number of iterations
            <0 : illegal input or breakdown

    Optional Inputs:

    x0  -- (0) default starting guess
    tol -- (1e-5) relative tolerance to achieve
    maxiter -- (10*n) maximum number of iterations

    """
    b = sb.asarray(b)
    n = len(b)
    typ = b.typecode()
    if maxiter is None:
        maxiter = n*10

    x = x0
    if x is None:
        x = sb.zeros(n,typ)
        
    matvec, psolve, rmatvec, rpsolve = (None,)*4
    ltr = _type_conv[typ]
    revcom = _iterative.__dict__[ltr+'bicgrevcom']
    stoptest = _iterative.__dict__[ltr+'stoptest2']

    resid = tol
    ndx1 = 1
    ndx2 = -1
    work = sb.zeros(6*n,typ)
    ijob = 1
    info = 0
    ftflag = True
    bnrm2 = -1.0
    iter_ = maxiter
    while 1:
        x, iter_, resid, info, ndx1, ndx2, sclr1, sclr2, ijob = \
           revcom(b, x, work, iter_, resid, info, ndx1, ndx2, ijob)
        slice1 = slice(ndx1-1, ndx1-1+n)
        slice2 = slice(ndx2-1, ndx2-1+n)
        if (ijob == -1):
            break
        elif (ijob == 1):
            if matvec is None:
                matvec = get_matvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*matvec(work[slice1])
        elif (ijob == 2):
            if rmatvec is None:
                rmatvec = get_rmatvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*rmatvec(work[slice1])
        elif (ijob == 3):
            if psolve is None:
                psolve = get_psolve(A)
            work[slice1] = psolve(work[slice2])
        elif (ijob == 4):
            if rpsolve is None:
                rpsolve = get_rpsolve(A)
            work[slice1] = rpsolve(work[slice2])
        elif (ijob == 5):
            if matvec is None:
                matvec = get_matvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*matvec(x)
        elif (ijob == 6):
            if ftflag:
                info = -1
                ftflag = False
            bnrm2, resid, info = stoptest(work[slice1], b, bnrm2, tol, info)
        ijob = 2

    return x, info

def bicgstab(A,b,x0=None,tol=1e-5,maxiter=None):
    """Use BIConjugate Gradient STABilized iteration to solve A x = b

    Inputs:

    A --   An array or an object with a matvec(x) method
           to represent A * x.  May also have a psolve(b) methods for
           representing solution to the preconditioning equation
           M * x = b.
    b --   An n-length vector

    Outputs:

    x  --  The converged solution
    info -- output result
            0  : successful exit
            >0 : convergence to tolerance not achieved, number of iterations
            <0 : illegal input or breakdown

    Optional Inputs:

    x0  -- (0) default starting guess
    tol -- (1e-5) relative tolerance to achieve
    maxiter -- (10*n) maximum number of iterations

    """
    b = sb.asarray(b)
    n = len(b)
    typ = b.typecode()
    if maxiter is None:
        maxiter = n*10

    x = x0
    if x is None:
        x = sb.zeros(n,typ)
        
    matvec, psolve = (None,)*2
    ltr = _type_conv[typ]
    revcom = _iterative.__dict__[ltr+'bicgstabrevcom']
    stoptest = _iterative.__dict__[ltr+'stoptest2']

    resid = tol
    ndx1 = 1
    ndx2 = -1
    work = sb.zeros(7*n,typ)
    ijob = 1
    info = 0
    ftflag = True
    bnrm2 = -1.0
    iter_ = maxiter
    while 1:
        x, iter_, resid, info, ndx1, ndx2, sclr1, sclr2, ijob = \
           revcom(b, x, work, iter_, resid, info, ndx1, ndx2, ijob)
        slice1 = slice(ndx1-1, ndx1-1+n)
        slice2 = slice(ndx2-1, ndx2-1+n)
        if (ijob == -1):
            break
        elif (ijob == 1):
            if matvec is None:
                matvec = get_matvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*matvec(work[slice1])
        elif (ijob == 2):
            if psolve is None:
                psolve = get_psolve(A)
            work[slice1] = psolve(work[slice2])
        elif (ijob == 3):
            if matvec is None:
                matvec = get_matvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*matvec(x)
        elif (ijob == 4):
            if ftflag:
                info = -1
                ftflag = False
            bnrm2, resid, info = stoptest(work[slice1], b, bnrm2, tol, info)
        ijob = 2

    return x, info

def cg(A,b,x0=None,tol=1e-5,maxiter=None):
    """Use Conjugate Gradient iteration to solve A x = b (A^H = A)

    Inputs:

    A --   An array or an object with a matvec(x) method
           to represent A * x.  May also have a psolve(b) methods for
           representing solution to the preconditioning equation
           M * x = b.
    b --   An n-length vector
    

    Outputs:

    x  --  The converged solution
    info -- output result
            0  : successful exit
            >0 : convergence to tolerance not achieved, number of iterations
            <0 : illegal input or breakdown

    Optional Inputs:

    x0  -- (0) default starting guess
    tol -- (1e-5) relative tolerance to achieve
    maxiter -- (10*n) maximum number of iterations

    """
    b = sb.asarray(b)
    n = len(b)
    typ = b.typecode()
    if maxiter is None:
        maxiter = n*10

    x = x0
    if x is None:
        x = sb.zeros(n,typ)
        
    matvec, psolve = (None,)*2
    ltr = _type_conv[typ]
    revcom = _iterative.__dict__[ltr+'cgrevcom']
    stoptest = _iterative.__dict__[ltr+'stoptest2']

    resid = tol
    ndx1 = 1
    ndx2 = -1
    work = sb.zeros(4*n,typ)
    ijob = 1
    info = 0
    ftflag = True
    bnrm2 = -1.0
    iter_ = maxiter
    while 1:
        x, iter_, resid, info, ndx1, ndx2, sclr1, sclr2, ijob = \
           revcom(b, x, work, iter_, resid, info, ndx1, ndx2, ijob)
        slice1 = slice(ndx1-1, ndx1-1+n)
        slice2 = slice(ndx2-1, ndx2-1+n)
        if (ijob == -1):
            break
        elif (ijob == 1):
            if matvec is None:
                matvec = get_matvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*matvec(work[slice1])
        elif (ijob == 2):
            if psolve is None:
                psolve = get_psolve(A)
            work[slice1] = psolve(work[slice2])
        elif (ijob == 3):
            if matvec is None:
                matvec = get_matvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*matvec(x)
        elif (ijob == 4):
            if ftflag:
                info = -1
                ftflag = False
            bnrm2, resid, info = stoptest(work[slice1], b, bnrm2, tol, info)
        ijob = 2

    return x, info


def cgs(A,b,x0=None,tol=1e-5,maxiter=None):
    """Use Conjugate Gradient Squared iteration to solve A x = b 

    Inputs:

    A --   An array or an object with a matvec(x) method
           to represent A * x.  May also have a psolve(b) methods for
           representing solution to the preconditioning equation
           M * x = b.
    b --   An n-length vector
    

    Outputs:

    x  --  The converged solution
    info -- output result
            0  : successful exit
            >0 : convergence to tolerance not achieved, number of iterations
            <0 : illegal input or breakdown

    Optional Inputs:

    x0  -- (0) default starting guess
    tol -- (1e-5) relative tolerance to achieve
    maxiter -- (10*n) maximum number of iterations

    """
    b = sb.asarray(b)
    n = len(b)
    typ = b.typecode()
    if maxiter is None:
        maxiter = n*10

    x = x0
    if x is None:
        x = sb.zeros(n,typ)
        
    matvec, psolve = (None,)*2
    ltr = _type_conv[typ]
    revcom = _iterative.__dict__[ltr+'cgsrevcom']
    stoptest = _iterative.__dict__[ltr+'stoptest2']

    resid = tol
    ndx1 = 1
    ndx2 = -1
    work = sb.zeros(7*n,typ)
    ijob = 1
    info = 0
    ftflag = True
    bnrm2 = -1.0
    iter_ = maxiter
    while 1:
        x, iter_, resid, info, ndx1, ndx2, sclr1, sclr2, ijob = \
           revcom(b, x, work, iter_, resid, info, ndx1, ndx2, ijob)
        slice1 = slice(ndx1-1, ndx1-1+n)
        slice2 = slice(ndx2-1, ndx2-1+n)
        if (ijob == -1):
            break
        elif (ijob == 1):
            if matvec is None:
                matvec = get_matvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*matvec(work[slice1])
        elif (ijob == 2):
            if psolve is None:
                psolve = get_psolve(A)
            work[slice1] = psolve(work[slice2])
        elif (ijob == 3):
            if matvec is None:
                matvec = get_matvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*matvec(x)
        elif (ijob == 4):
            if ftflag:
                info = -1
                ftflag = False
            bnrm2, resid, info = stoptest(work[slice1], b, bnrm2, tol, info)
        ijob = 2

    return x, info

# not working yet.
def gmres(A,b,restrt=None,x0=None,tol=1e-5,maxiter=None):
    """Use Generalized Minimal RESidual iteration to solve A x = b 

    Inputs:

    A --   An array or an object with a matvec(x) method
           to represent A * x.  May also have a psolve(b) methods for
           representing solution to the preconditioning equation
           M * x = b.
    b --   An n-length vector
    

    Outputs:
    
    x  --  The converged solution
    info -- output result
            0  : successful exit
            >0 : convergence to tolerance not achieved, number of iterations
            <0 : illegal input or breakdown

    Optional Inputs:

    restrt -- (n) When to restart (change this to get faster performance -- but
                   may not converge). 
    x0  -- (0) default starting guess
    tol -- (1e-5) relative tolerance to achieve
    maxiter -- (10*n) maximum number of iterations

    """
    b = sb.asarray(b)
    n = len(b)
    typ = b.typecode()
    if maxiter is None:
        maxiter = n*10

    x = x0
    if x is None:
        x = sb.zeros(n,typ)
        
    matvec, psolve = (None,)*2
    ltr = _type_conv[typ]
    revcom = _iterative.__dict__[ltr+'gmresrevcom']
    stoptest = _iterative.__dict__[ltr+'stoptest2']

    if restrt is None:
        restrt = n
    resid = tol
    ndx1 = 1
    ndx2 = -1
    work = sb.zeros((6+restrt)*n,typ)
    work2 = sb.zeros((restrt+1)*(2*restrt+2),typ)
    ijob = 1
    info = 0
    ftflag = True
    bnrm2 = -1.0
    iter_ = maxiter
    while 1:
        x, iter_, resid, info, ndx1, ndx2, sclr1, sclr2, ijob = \
           revcom(b, x, restrt, work, work2, iter_, resid, info, ndx1, ndx2, ijob)
        slice1 = slice(ndx1-1, ndx1-1+n)
        slice2 = slice(ndx2-1, ndx2-1+n)
        if (ijob == -1):
            break
        elif (ijob == 1):
            if matvec is None:
                matvec = get_matvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*matvec(x)            
        elif (ijob == 2):
            if psolve is None:
                psolve = get_psolve(A)
            work[slice1] = psolve(work[slice2])
        elif (ijob == 3):
            if matvec is None:
                matvec = get_matvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*matvec(work[slice1])
        elif (ijob == 4):
            if ftflag:
                info = -1
                ftflag = False
            bnrm2, resid, info = stoptest(work[slice1], b, bnrm2, tol, info)
        ijob = 2

    return x, info


def qmr(A,b,x0=None,tol=1e-5,maxiter=None):
    """Use Quasi-Minimal Residucal iteration to solve A x = b

    Inputs:

    A --   An array or an object with matvec(x) and rmatvec(x) methods
           to represent A * x and A^H * x respectively.  May also have
           psolve(b,<which>) and rpsolve(b,<which>) methods for
           representing solutions to the preconditioning equations
           M * x = b and M^H * x = b respectively.   The <which> argument
           may be given to specify 'left' or 'right' preconditioning.
    b --   An n-length vector

    Outputs:

    x  --  The converged solution
    info -- output result
            0  : successful exit
            >0 : convergence to tolerance not achieved, number of iterations
            <0 : illegal input or breakdown

    Optional Inputs:

    x0  -- (0) default starting guess
    tol -- (1e-5) relative tolerance to achieve
    maxiter -- (10*n) maximum number of iterations

    """
    b = sb.asarray(b)
    n = len(b)
    typ = b.typecode()
    if maxiter is None:
        maxiter = n*10

    x = x0
    if x is None:
        x = sb.zeros(n,typ)
        
    matvec, psolve, rmatvec, rpsolve = (None,)*4
    ltr = _type_conv[typ]
    revcom = _iterative.__dict__[ltr+'qmrrevcom']
    stoptest = _iterative.__dict__[ltr+'stoptest2']

    resid = tol
    ndx1 = 1
    ndx2 = -1
    work = sb.zeros(11*n,typ)
    ijob = 1
    info = 0
    ftflag = True
    bnrm2 = -1.0
    iter_ = maxiter
    while 1:
        x, iter_, resid, info, ndx1, ndx2, sclr1, sclr2, ijob = \
           revcom(b, x, work, iter_, resid, info, ndx1, ndx2, ijob)
        slice1 = slice(ndx1-1, ndx1-1+n)
        slice2 = slice(ndx2-1, ndx2-1+n)
        if (ijob == -1):
            break
        elif (ijob == 1):
            if matvec is None:
                matvec = get_matvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*matvec(work[slice1])
        elif (ijob == 2):
            if rmatvec is None:
                rmatvec = get_rmatvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*rmatvec(work[slice1])
        elif (ijob == 3):
            if psolve is None:
                psolve = get_psolveq(A)
            work[slice1] = psolve(work[slice2],'left')
        elif (ijob == 4):
            if psolve is None:
                psolve = get_psolveq(A)
            work[slice1] = psolve(work[slice2],'right')            
        elif (ijob == 5):
            if rpsolve is None:
                rpsolve = get_rpsolveq(A)
            work[slice1] = rpsolve(work[slice2],'left')
        elif (ijob == 6):
            if rpsolve is None:
                rpsolve = get_rpsolveq(A)
            work[slice1] = rpsolve(work[slice2],'right')
        elif (ijob == 7):
            if matvec is None:
                matvec = get_matvec(A)
            work[slice2] *= sclr2
            work[slice2] += sclr1*matvec(x)
        elif (ijob == 8):
            if ftflag:
                info = -1
                ftflag = False
            bnrm2, resid, info = stoptest(work[slice1], b, bnrm2, tol, info)
        ijob = 2

    return x, info





import unittest
from Numeric import *

from scipy_test.testing import *
set_package_path()
from weave import inline_tools
restore_path()
set_local_path()
from test_scxx import *
restore_path()

class test_inline(unittest.TestCase):
    """ These are long running tests...
    
         I'd like to benchmark these things somehow.
    """
    def check_exceptions(self,level=5):
        a = 3                                  
        code = """
               if (a < 2)
                  throw_error(PyExc_ValueError,
                              "the variable 'a' should not be less than 2");
               else    
                   return_val = PyInt_FromLong(a+1);
               """
        result = inline_tools.inline(code,['a'])
        assert(result == 4)
        
        try:
            a = 1
            result = inline_tools.inline(code,['a'])
            assert(1) # should've thrown a ValueError
        except ValueError:
            pass
        
        from distutils.errors import DistutilsError, CompileError    
        try:
            a = 'string'
            result = inline_tools.inline(code,['a'])
            assert(1) # should've gotten an error
        except: 
            # ?CompileError is the error reported, but catching it doesn't work
            pass

if __name__ == "__main__":
    ScipyTest('weave.inline_tools').run()
